import java.util.*;

public class RoutePlanner {

    public static final double SLOW_COST = 2.0;

    /**
     * Use approximation in A*
     */
    public boolean doApproximation;

    /**
     * The FactoryMap
     */
    private FactoryMap factoryMap;

    /**
     * GraphNode factory to create GraphNode objects
     */
    private GraphNode.GraphNodeCreator nodeCreator;

    /**
     * The currently selected node
     */
    private GraphNode currentPosition;

    /**
     * The target node
     */
    private GraphNode targetPosition;

    //TODO: Add your datastructures as fields

    /**
     * Initializes the RoutePlanner for running the A*-Algorithm steps with the doNextStep() method and marks the factoryMap accordingly
     *
     * @param factoryMap      the map to find the shortest path on
     * @param start           the startpoint of the path
     * @param target          the tragetpoint to find the shortest path to
     * @param doApproximation indicates if the approximation of A* should be used
     */
    public RoutePlanner(FactoryMap factoryMap, FactoryMap.MapPoint start, FactoryMap.MapPoint target, boolean doApproximation) {
        //TODO
    }

    public RoutePlanner(FactoryMap factoryMap, FactoryMap.MapPoint start, FactoryMap.MapPoint target) {
        this(factoryMap, start, target, true);
    }


    /**
     * Updates the current position with the provided GraphNode
     * and updates the Points on the factoryMap.
     *
     * @param newNode
     */
    private void updateCurrentPosition(GraphNode newNode) {
        currentPosition.getPoint().setExplored();
        newNode.getPoint().setCurrent();
        this.currentPosition = newNode;
    }

    /**
     * Performs one round of the A*-Algorithm.
     * In each round, the algorithm first explores the neighboring nodes of the current node.
     * In the exploration step, the adjacent nodes are updated with new distance information/previous nodes and
     * the data structures are updated accordingly.
     * See https://de.wikipedia.org/wiki/A*-Algorithmus for more details.
     * As a last step, the algorithm selects a new current node from the "open nodes" data structure
     * by choosing the node with the smallest f-value.
     * The algorithm finishes if the current node equals the target node.
     * The shortest path can then be retrieved by calling the getShortestPath() method
     *
     * @return true if the algorithm has finished (i.e., currentPosition equals targetPosition) false if not
     * @throws RoutePlannerException if there is no valid path to the target
     */
    public boolean doNextStep() throws RoutePlannerException {
        if (currentPosition.equals(targetPosition))
            return true;
        //TODO
        return false;
    }

    /**
     * Returns the shortest path if the algorithm has finished.
     * Note that this method can be called more than once and should return the correct shortest 
     * path on each invocation.
     * @throws RoutePlannerException if the algorithm has not finished yet.
     */
    public List<FactoryMap.MapPoint> getShortestPath() throws RoutePlannerException {
        if (!currentPosition.equals(targetPosition))
            throw new RoutePlannerException("Target not yet found");
        //TODO
        return null;
    }


    public FactoryMap getFactoryMap() {
        return factoryMap;
    }

    public GraphNode getCurrentPosition() {
        return currentPosition;
    }
}
