import java.io.PrintStream;
import java.util.List;

public class RouteUtil {


    /**
     * Computes the shortest path and writes the intermediate maps to out
     * @param planner
     * @param out
     * @param printFrequency
     * @throws RoutePlannerException
     */
    public static int computeShortestPath(RoutePlanner planner, PrintStream out, int printFrequency) throws RoutePlannerException {
        if (out != null)
            out.println(planner.getFactoryMap());
        int count = 0;
        while (!planner.doNextStep()) {
            if (++count % printFrequency == 0)
                if (out != null)
                    out.println(planner.getFactoryMap());
        }

        List<FactoryMap.MapPoint> path = planner.getShortestPath();
        for (FactoryMap.MapPoint point : path) {
            point.setPath();
        }
        if (out != null)
            out.println(planner.getFactoryMap());
        return count;
    }

    public static double computeDistance(List<FactoryMap.MapPoint> path) {
        double dist = 0;
        for (int i = 0; i < path.size() - 1; i++) {
            dist += path.get(i).getDistanceWithSlowingFactor(path.get(i + 1), RoutePlanner.SLOW_COST);
        }
        return dist;
    }
}
